from typing import Any

from fastapi import APIRouter
from tompy.datetime import mstime

from otlpy.model.v1 import (
    INTERVAL,
    ORDER_SIDE,
    ORDER_TYPE,
    PQ,
    Buy,
    Cancel,
    Candlestick,
    InBuy,
    InCancel,
    InOrders,
    InSell,
    OrderBook,
    Sell,
)

v1 = APIRouter(prefix="/api/v1")


@v1.get("/market/orderbook", tags=["Market"])
async def orderbook(ticker: str) -> OrderBook:
    return OrderBook(
        mstime=mstime(),
        ticker=ticker,
        bids=[PQ(p="100", q="1")],
        asks=[PQ(p="101", q="1")],
    )


@v1.get("/market/candlesticks", tags=["Market"])
async def candlesticks(
    ticker: str, interval: INTERVAL = INTERVAL.D1
) -> list[Candlestick]:
    return [
        Candlestick(
            ot=mstime(),
            o="100",
            h="101",
            l="99",
            c="100",
            v="10",
            ct=mstime(),
        ),
    ]


@v1.post("/order/buy", tags=["Order"])
async def buy(data: InBuy) -> Buy:
    return Buy(
        oside=ORDER_SIDE.BUY,
        otype=data.otype,
        ticker=data.ticker,
        qty=data.qty,
        price=data.price,
        comment=data.comment,
    )


@v1.post("/order/sell", tags=["Order"])
async def sell(data: InSell) -> Sell:
    return Sell(
        oside=ORDER_SIDE.SELL,
        otype=data.otype,
        ticker=data.ticker,
        qty=data.qty,
        price=data.price,
        comment=data.comment,
    )


@v1.post("/order/cancel", tags=["Order"])
async def cancel(data: InCancel) -> Cancel:
    return Cancel(
        oside=data.origin.oside,
        otype=ORDER_TYPE.LIMIT,
        ticker=data.origin.ticker,
        qty=data.origin.qty,
        price="0",
        comment=data.comment,
        origin=data.origin,
    )


@v1.post("/order/orders", tags=["Order"])
async def orders(data: InOrders) -> list[Any]:
    return []
